package com.trafficparrot.sdk.handlebars;

import com.github.jknack.handlebars.Handlebars;
import com.github.jknack.handlebars.Helper;
import com.github.jknack.handlebars.helper.StringHelpers;
import com.trafficparrot.sdk.jms.MissingHelper;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static java.util.Arrays.asList;
import static java.util.Collections.emptyList;
import static org.apache.commons.lang3.StringUtils.isBlank;

public class HandlebarsFactory {

    public static Handlebars handlebars(String commaSeparatedHandlebarHelpers) {
        return handlebars(toHelpers(parseList(commaSeparatedHandlebarHelpers)));
    }

    private static Handlebars handlebars(Map<String, Helper> helpers) {
        Handlebars handlebars = new Handlebars();

        for (StringHelpers helper : StringHelpers.values()) {
            handlebars.registerHelper(helper.name(), helper);
        }

        for (Map.Entry<String, Helper> entry : helpers.entrySet()) {
            handlebars.registerHelper(entry.getKey(), entry.getValue());
        }

        handlebars.registerHelperMissing(new MissingHelper());
        return handlebars;
    }

    // TODO(4/5) copied from VirtualServiceServer
    private static Map<String, Helper> toHelpers(List<String> dynamicResponseTemplateExtensions) {
        Map<String, Helper> helpers = new HashMap<>();
        for (String extension : dynamicResponseTemplateExtensions) {
            TrafficParrotHelper helper = toHelper(extension);
            helpers.put(helper.getName(), helper);
        }
        return helpers;
    }

    // TODO(4/5) copied from VirtualServiceServer
    private static TrafficParrotHelper toHelper(String helperClassName) {
        try {
            return ((TrafficParrotHelper) Class.forName(helperClassName).newInstance());
        } catch (ClassNotFoundException | InstantiationException | IllegalAccessException e) {
            throw new IllegalArgumentException(e);
        }
    }

    // TODO(4/5) copied from VirtualServiceServer
    public static List<String> parseList(String propertyValue) {
        if (isBlank(propertyValue)) {
            return emptyList();
        }
        return asList(propertyValue.split(","));
    }
}
