package com.trafficparrot.sdk.jms;

import com.github.jknack.handlebars.Handlebars;
import com.github.jknack.handlebars.Template;

import java.io.IOException;

import static com.trafficparrot.sdk.handlebars.HandlebarsFactory.handlebars;
import static java.lang.String.format;
import static java.util.Collections.emptyMap;

public class JmsHandlebarsResponseTransformer {
    private final Handlebars handlebars;
    private final boolean enabled;
    private final TpLogger logger;

    public JmsHandlebarsResponseTransformer(Handlebars handlebars, boolean enabled, TpLogger logger) {
        this.handlebars = handlebars;
        this.enabled = enabled;
        this.logger = logger;
    }

    public static JmsHandlebarsResponseTransformer handlebarsTransformer(String commaSeparatedHandlebarHelpers, Boolean handlebarsEnabled, TpLogger logger) {
        if (handlebarsEnabled) {
            return defineHelpers(logger, handlebars(commaSeparatedHandlebarHelpers));
        }
        return handlebarsAreDisabled(logger);
    }

    public static JmsHandlebarsResponseTransformer handlebarsAreDisabled(TpLogger logger) {
        return new JmsHandlebarsResponseTransformer(null, false, logger);
    }

    private static JmsHandlebarsResponseTransformer defineHelpers(TpLogger logger, Handlebars handlebars) {
        return new JmsHandlebarsResponseTransformer(handlebars, true, logger);
    }

    public JmsResponse transform(JmsResponse response) {
        if (enabled) {
            try {
                Template responseTemplate = handlebars.compileInline(response.text);
                String responseTextWithTemplateApplied = responseTemplate.apply(emptyMap());
                return new JmsResponse(response.destination, response.jmsResponseTransformerClassName, responseTextWithTemplateApplied);
            } catch (com.github.jknack.handlebars.HandlebarsException | IOException e) {
                String errorMessage = format("Problem while applying handlebars: \n\n%s\n\nOriginal text: \n%s", e.getMessage(), response.text);
                logger.error(errorMessage, e);
                return new JmsResponse(response.destination, errorMessage + ". \n\nSee logs for more details.");
            }
        } else {
            return response;
        }
    }
}
