package com.trafficparrot.sdk.jms;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.collect.ComparisonChain;

import static org.apache.commons.lang3.builder.ToStringBuilder.reflectionToString;
import static org.apache.commons.lang3.builder.ToStringStyle.SHORT_PREFIX_STYLE;

public class JmsResponse implements Comparable<JmsResponse> {
    public static final String NO_TRANSFORMER = "NO_TRANSFORMER";

    public final Destination destination;
    public final String jmsResponseTransformerClassName;
    public final String text;

    public JmsResponse(@JsonProperty("destination") Destination destination,
                       @JsonProperty("jmsResponseTransformerClassName") String jmsResponseTransformerClassName,
                       @JsonProperty("text") String text) {
        this.destination = destination;
        this.jmsResponseTransformerClassName = jmsResponseTransformerClassName;
        this.text = text;
    }

    public JmsResponse(Destination destination, String text) {
        this.destination = destination;
        this.jmsResponseTransformerClassName = NO_TRANSFORMER;
        this.text = text;
    }

    @Override
    @JsonIgnore
    public String toString() {
        return reflectionToString(this, SHORT_PREFIX_STYLE);
    }

    @Override
    public int compareTo(JmsResponse other) {
        return ComparisonChain.start()
                .compare(destination, other.destination)
                .compare(text, other.text)
                .result();
    }
}
