package com.trafficparrot.sdk.jms;

import javax.jms.Message;

import static com.trafficparrot.sdk.jms.JmsResponse.*;
import static java.lang.String.format;

public abstract class JmsResponseTransformer {
    protected final TpLogger logger;

    public JmsResponseTransformer() {
        this(new Slf4jTpLogger(JmsResponseTransformer.class));
    }

    public JmsResponseTransformer(TpLogger logger) {
        this.logger = logger;
    }

    public final JmsResponse transform(Destination requestDestination, Message requestMessage, JmsResponse response) {
        try {
            JmsResponse jmsResponse = doTransform(requestDestination, requestMessage, response);
            if (!NO_TRANSFORMER.equals(jmsResponse.jmsResponseTransformerClassName)) {
                String errorMessage = format("Please set the jmsResponseTransformerClassName to %s in %s (it was set to %s)", NO_TRANSFORMER, getClass().getCanonicalName(), jmsResponse.jmsResponseTransformerClassName);
                logger.error(errorMessage);
                return new JmsResponse(response.destination,  errorMessage);
            }
            return jmsResponse;
        } catch (Exception e) {
            String errorMessage = format("%s had a problem while transforming: %s", getClass().getCanonicalName(), e.getMessage());
            logger.error(errorMessage, e);
            return new JmsResponse(response.destination, errorMessage + ". See logs for more details.");
        }
    }

    protected abstract JmsResponse doTransform(Destination requestDestination, Message requestMessage, JmsResponse response) throws Exception;

    public String getDisplayName() {
        return getClass().getSimpleName();
    }
}
